# License: BSD 3 clause
'''
This module exists solely for version information so I only have to change it
in one place. Based on the suggestion `here. <http://bit.ly/16LbuJF>`_

:author: Dan Blanchard (dblanchard@ets.org)
:organization: ETS
'''

__version__ = '0.22.4'
VERSION = tuple(int(x) for x in __version__.split('.'))
